#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include <libusb-1.0/libusb.h>

void error(const char* s) {
	printf("%s\n", s);
	libusb_exit(NULL);
	exit(1);
}

//#define DEBUG	1

#define USB_REQ_DFU_DNLOAD	1
#define BULK_SIZE	4096
#define MAX_FILE_SIZE 2000000
unsigned char firmware_buf[MAX_FILE_SIZE];

void do_upload() {
	struct libusb_device_handle *devh = NULL;
	int size = 0;
	char *buf = firmware_buf;
	
	if (libusb_init(NULL))
		error("Unable to initialize libusb");

#ifdef DEBUG
	libusb_set_debug(NULL, 3);
#endif
	
	FILE *f = fopen("ath3k-1.fw", "rb");
	if (!f)
		error("Unable to find file ath3k-1.fw");

	size = fread(buf, 1, MAX_FILE_SIZE, f);
//	printf("%d\n", size);
	if (size <= 0 || size >= MAX_FILE_SIZE)
		error("Invalid firmware file");
		
	devh = libusb_open_device_with_vid_pid(NULL, 0x311d, 0x03f0);
	if (devh <= 0)
		error("Atheros AR3011 not found");
	
	if (libusb_claim_interface(devh, 0) < 0)
		error("Unable to claim interface");
	
	
	if (libusb_control_transfer(devh, LIBUSB_REQUEST_TYPE_VENDOR, USB_REQ_DFU_DNLOAD, 
								0, 0, buf, 20, 10) < 0)
		error("Can't initiate firmware transfer");


	printf("Uploading firmware...");
	
	buf += 20;
	size -= 20;
	
	while (size) {
		int to_send = size < BULK_SIZE ? size : BULK_SIZE; 
		int sent, res;
		
		res = libusb_bulk_transfer(devh, 0x02, buf, to_send, &sent, 3000);
//		printf("%d %d %d\n", res, to_send, sent);

		if (res < 0 || to_send != sent)
			error("Firmware transfer failed");
		
		printf(".");
		buf += sent;
		size -= sent;
	}
	printf("done!\n");

	libusb_release_interface(devh, 0);
	libusb_exit(NULL);
}

int main(void)
{
	do_upload();
}
